CREATE TABLE IF NOT EXISTS subjects (
  id           SERIAL PRIMARY KEY,
  login        TEXT UNIQUE,
  is_group     BOOLEAN,
  is_remote    BOOLEAN,
  is_superuser BOOLEAN,
  display_name TEXT,
  email        TEXT,
  is_revoked   BOOLEAN DEFAULT FALSE,
  last_login   TIMESTAMP,
  password     TEXT,
  reset_password_uuid UUID,
  failed_login_attempts INTEGER DEFAULT 0
);
--;;

CREATE TABLE IF NOT EXISTS groupings (
  -- Group ID
  gid INTEGER REFERENCES subjects (id) ON DELETE CASCADE,

  -- Remote User ID
  ruid INTEGER REFERENCES subjects (id) ON DELETE CASCADE,

 CONSTRAINT groupings_pkey PRIMARY KEY (gid, ruid)
);
--;;

CREATE TABLE IF NOT EXISTS roles (
  id           SERIAL PRIMARY KEY,
  display_name TEXT UNIQUE,
  description  TEXT
)
--;;

CREATE TABLE IF NOT EXISTS object_types (
       object_type  TEXT PRIMARY KEY,
       display_name TEXT UNIQUE,
       description  TEXT
);
--;;

CREATE TABLE IF NOT EXISTS actions (
       name          TEXT,
       display_name  TEXT,
       description   TEXT,
       object_type   TEXT REFERENCES object_types ON DELETE CASCADE,
       has_instances BOOLEAN,
       PRIMARY KEY   (name, object_type)
);
--;;

CREATE TABLE IF NOT EXISTS permissions (
       id          SERIAL PRIMARY KEY,
       object_type TEXT,
       action      TEXT,
       instance    TEXT NOT NULL DEFAULT '*',

       FOREIGN KEY (object_type, action) REFERENCES actions (object_type, name) MATCH FULL ON DELETE CASCADE,
       CONSTRAINT permissions_triple UNIQUE (object_type, action, instance)
);
--;;

CREATE TABLE IF NOT EXISTS subject_roles (
  sid       INTEGER REFERENCES subjects (id) ON DELETE CASCADE,
  rid       INTEGER REFERENCES roles (id) ON DELETE CASCADE,
  CONSTRAINT user_roles_pkey PRIMARY KEY (sid, rid)
);
--;;

CREATE TABLE IF NOT EXISTS roles_permissions (
  pid INTEGER REFERENCES permissions (id) ON DELETE CASCADE,
  rid INTEGER REFERENCES roles (id) ON DELETE CASCADE,
  CONSTRAINT roles_permissions_pkey PRIMARY KEY (pid, rid)
);
--;;

CREATE TABLE IF NOT EXISTS directory_settings (
  id INTEGER PRIMARY KEY, -- not serial because singleton table

  display_name           TEXT,
  help_link              TEXT,
  type                   TEXT,

  hostname               TEXT,
  port                   INTEGER,
  ssl                    BOOLEAN,
  login                  TEXT,
  password               TEXT,
  connect_timeout        INTEGER,

  base_dn                TEXT,

  user_rdn                TEXT,
  user_display_name_attr TEXT,
  user_email_attr        TEXT,
  user_lookup_attr       TEXT,

  group_rdn               TEXT,
  group_object_class     TEXT,
  group_name_attr        TEXT,
  group_member_attr      TEXT,
  group_lookup_attr      TEXT
);
